# Generated with ChatGPT

from .base import MessageSchedule as MS

__ignore__ = True

GOOD_DAY_NIGHT = [
    MS(
        (
            "早上好😃",
            "早啊大佬们",
            "大佬们早上好",
            "起床了 有精神啊",
            "今天好心情",
            "早上好呀！",
            "起床了 先到群里签个到",
            "各位早上好啊",
            "大家早上好！祝大家一天愉快！",
            "祝大家有美好的一天~",
        ),
        ("7:00", "9:00"),
    ),
    MS(("午安", "午安 大家", "午安 xdm", "午安 各位"), ("1:00", "1:40"), possibility=0.6),
    MS(("下班了，吃饭了", "下班!摸鱼的一天结束了"), ("18:00", "18:05"), only="weekdays"),
    MS(("晚安了各位", "晚安，好梦", "晚安，玛卡巴卡", "晚安，兄弟们", "睡觉了，老哥们", "各位大佬早点休息呀", "晚安，各位大佬", "晚安"), ("23:00", "1:00")),
]


def WATERY(n, *args, **kw):
    return [
        MS(
            (
                "手动签",
                "水一分",
                "水一哈",
                "签到",
                "打卡",
                "水一水，提提神",
                "想低调当个大水逼",
                "水来",
                "好人一生平安！",
                "水 爽翻",
                "天天签到赚积分",
                "来这里水一水，拿点积分走人啦！",
                "玛卡巴卡巴卡",
                "听说这里有积分奖励",
                "好无聊啊最近",
                "好累啊，想放假",
                "有啥好玩的吗",
                "最近有什么好看的电影吗",
                "水水水",
                "我觉得这里针不戳",
                "来水一发",
                "今天的水量还不够",
                "水一水解乏",
                "我要水爆这里",
                "水个存在感",
                "偷偷来水一下",
                "来水一波，积分到手",
                "水不停，潮流不断",
                "水一发，走天下",
                "水不怕多，只怕少",
                "热爱水的人总有积分",
                "大水不流空，积分不来无",
                "水不只是水，更是一种态度",
                "水是一种技术活",
                "别闲着，快来水",
                "无聊就来水一下",
                "水的多，想的少",
                "水是心灵的抚慰 水是积分的保障 水是无边的欢乐",
                "水之所在，积分自来",
                "水之不竭，积分之源",
                "水到渠成，积分源源不断",
                "天天水，不用愁",
                "水不止五分钟，积分不止一点",
                "水不用解释，水就是水",
                "我水，故我在",
                "生命在于水，积分在于水",
                "水，让生命更加美好",
                "今天又水一水~",
            ),
            *args,
            **kw,
        )
    ] * n
